/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include "persistencemanager.h"
#include "sqlagent.h"
#include "global.h"
#include <QSettings>
#include <QFile>
#include <QDir>

bool Persistence::Manager::existsConfig()
{
    QSettings setting(ORGANIZATION_NAME, APPLICATION_NAME);

    return setting.value("Executed", false).toBool();
}

bool Persistence::Manager::createConfig(bool overwrite)
{
    if(existsConfig() && !overwrite)
        return false;

    QSettings setting(ORGANIZATION_NAME, APPLICATION_NAME);

    setting.setValue("Executed",                  true);
    setting.beginGroup("Application");
    setting.setValue("Currency",                  DEFAULT_APPLICATION_CURRENCY);
    setting.beginGroup("Precision");
    setting.setValue("Money",                     DEFAULT_APPLICATION_MONEY_PRECISION);
    setting.endGroup();
    setting.endGroup();
    setting.beginGroup("Storage");
    setting.setValue("Type",                      DEFAULT_STORAGE_TYPE);
    setting.setValue("Path",                      DEFAULT_STORAGE_PATH);
    setting.beginGroup("DBMS");
    setting.setValue("Driver",                    DEFAULT_STORAGE_DBMS_DRIVER);
    setting.setValue("Name",                      DEFAULT_STORAGE_DBMS_NAME);
    setting.setValue("Host",                      DEFAULT_STORAGE_DBMS_HOST);
    setting.setValue("Port",                      DEFAULT_STORAGE_DBMS_PORT);
    setting.setValue("User",                      DEFAULT_STORAGE_DBMS_USER);
    setting.setValue("Pass",                      DEFAULT_STORAGE_DBMS_PASS);
    setting.endGroup();
    setting.endGroup();

    return true;
}

bool Persistence::Manager::deleteConfig()
{
    QSettings setting(ORGANIZATION_NAME, APPLICATION_NAME);

    if(!existsConfig())
        return false;

    setting.clear();

    return true;
}

QVariant Persistence::Manager::readConfig(const QString &key, const QString &group)
{
    QSettings setting(ORGANIZATION_NAME, APPLICATION_NAME);

    if(existsConfig()) {
        setting.beginGroup(group);
        return setting.value(key);
    }

    return QVariant();
}

bool Persistence::Manager::writeConfig(const QVariant &value, const QString &key, const QString &group)
{
    QSettings setting(ORGANIZATION_NAME, APPLICATION_NAME);

    if(!existsConfig())
        return false;

    setting.beginGroup(group);
    if(!setting.contains(key))
        return false;
    setting.setValue(key, value);
    setting.endGroup();

    return true;
}

bool Persistence::Manager::existsStorage()
{
    if(!existsConfig())
        return false;

    switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
    case SQLITE:
        QString driver = readConfig("Driver", "Storage/DBMS").toString();
        if(driver == "QSQLITE")
            return QFile(readConfig("Name", "Storage/DBMS").toString()).exists();
        break;
    }

    return false;
}

bool Persistence::Manager::createStorage(bool overwrite)
{
    if(!existsConfig() || (existsStorage() && !overwrite))
        return false;

    QDir storagePath(readConfig("Path", "Storage").toString());;

    if(!storagePath.exists() && !storagePath.mkpath(storagePath.path()))
            return false;

    switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
    case SQLITE:
        return createSQLiteSchema();
    }

    return false;
}

bool Persistence::Manager::deleteStorage()
{
    if(!existsStorage())
        return false;

    switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
    case SQLITE:
        return QFile::remove(readConfig("Name", "Storage/DBMS").toString());
    }

    return false;
}

bool Persistence::Manager::createSQLiteSchema()
{
    SQLAgent *agent = SQLAgent::instance();

    if(!agent -> connect())
        return false;

    bool regist = agent -> create("CREATE TABLE IF NOT EXISTS regist (\n"
                                  "    id        INTEGER,\n"
                                  "    date      TEXT,\n"
                                  "    time      TEXT,\n"
                                  "    lecti     INTEGER,\n"
                                  "    lectf     INTEGER,\n"
                                  "    kwh       INTEGER,\n"
                                  "    import    REAL,\n"
                                  "    CONSTRAINT regist_pk_ct  PRIMARY KEY(id)\n"
                                  ")");

    agent -> disconnect();

    return regist;
}


bool Persistence::Manager::connectStorage()
{
    bool ok = false;

    try {
        Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
        switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
        case SQLITE:
            ok = agent -> connect() && agent -> create("PRAGMA foreign_keys=ON;");
            break;
        }
    } catch(Persistence::SQLAgentException sqlException) {}

    return ok;
}

void Persistence::Manager::disconnectStorage()
{
    try {
        Persistence::SQLAgent *agent = Persistence::SQLAgent::instance();
        agent -> disconnect();
        delete agent;
    } catch(Persistence::SQLAgentException sqlException) {}
}

bool Persistence::Manager::importStorage(const QString &fileName)
{
    switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
    case SQLITE:
        QFile storage(readConfig("Name", "Storage/DBMS").toString());

        if(storage.remove())
            return QFile(fileName).copy(readConfig("Name", "Storage/DBMS").toString());
    }

    return false;
}

bool Persistence::Manager::exportStorage(const QString &fileName)
{
    switch(static_cast<DBMSType>(readConfig("Type", "Storage").toInt())) {
    case SQLITE:
        QFile file(fileName);

        if(file.exists() && !file.remove())
                return false;

        return QFile(readConfig("Name", "Storage/DBMS").toString()).copy(fileName);
    }

    return false;
}
