/**
 *  This file is part of RegCEl.
 *
 *  Copyright (c) 2012 Yunior Barceló Chávez barceloch@gmail.com
 *
 *  RegCEl is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  RegCEl is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with RegCEl.  If not, see <http://www.gnu.org/licenses/>.
 *
 **/

#include <QtGui>
#include "mainwindow.h"
#include "registmanager.h"
#include "persistencemanager.h"
#include "registdialog.h"
#include "regcelwidget.h"
#include "global.h"

View::MainWindow::MainWindow()
{

    createWidgets();
    createConnections();
    setWindowIcon(QIcon(":/images/regcel.png"));
    setWindowTitle(trUtf8("RegCEl 0.1.0"));
    resize(585, 600);

    _connected = false;
    connectStorage();
    manageRegist();

}

View::MainWindow::~MainWindow()
{
    disconnectStorage();

}

void View::MainWindow::closeEvent(QCloseEvent *event)
{
    if(verifyExit()){
        event -> accept();
    } else
        event -> ignore();
}



void View::MainWindow::connectStorage()
{
    if(!(_connected = Persistence::Manager::connectStorage()))
        QMessageBox::critical(this, trUtf8("Conectar Almacenamiento"),
                                    trUtf8("Hubo un error durante el intento de conexión.\n"
                                           "Compruebe su configuración de almacenamiento."),
                                    QMessageBox::Ok);

    setStorageConnected(_connected);
}

void View::MainWindow::disconnectStorage()
{
    Persistence::Manager::disconnectStorage();
    setStorageConnected((_connected = false));


}

void View::MainWindow::importStorage()
{
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    trUtf8("Importar Almacenamiento"),
                                                    QDesktopServices::storageLocation(QDesktopServices::HomeLocation),
                                                    trUtf8("Todos los Archivos (*.*)"));

    if(fileName.isEmpty() || !verifyImportStorage())
        return;

    QApplication::setOverrideCursor(Qt::WaitCursor);
    bool ok = Persistence::Manager::importStorage(fileName);
    QApplication::restoreOverrideCursor();

    if(!ok)
        QMessageBox::warning(this, trUtf8("Importar Almacenamiento"),
                                   trUtf8("Hubo errores durante la importación.\n"
                                          "Verifique si el fichero existe y/o los permisos del directorio."),
                                   QMessageBox::Ok);
    else
        statusBar() -> showMessage(trUtf8("Importación Completada"), 5000);
}

void View::MainWindow::exportStorage()
{
    QString fileName = QFileDialog::getSaveFileName(this, trUtf8("Exportar Almacenamiento"),
                                                QDesktopServices::storageLocation(QDesktopServices::HomeLocation),
                                                trUtf8("Todos los Archivos (*.*)"));

    if(fileName.isEmpty())
        return;

    QApplication::setOverrideCursor(Qt::WaitCursor);
    bool ok = Persistence::Manager::exportStorage(fileName);
    QApplication::restoreOverrideCursor();

    if(!ok)
        QMessageBox::warning(this, trUtf8("Exportar Almacenamiento"),
                                   trUtf8("Hubo errores durante la exportación.\n"
                                          "Verifique los permisos del directorio destino."),
                                   QMessageBox::Ok);
    else
        statusBar() -> showMessage(trUtf8("Exportación Completada"), 5000);
}


void View::MainWindow::manageRegist()
{
   //_productEditor = new View::Management::ProductEditor;
    _regcelWidget = new View::Management::RegCEl;
    setCentralWidget(_regcelWidget);
}

void View::MainWindow::about()
{
    QMessageBox::about(this, trUtf8("RegCEl 0.1.0"),
            trUtf8("<h2>RegCEl 0.1.0</h2>"
            "<h3>Registro del Consumo Eléctico</h3>"
            "<p>(C) 2012 Yunior Barceló Chávez <a href= \"mailto:barceloch@gmail.com\" >barceloch@gmail.com</a></p>"
            "<p><a href= \http://gutl.jovenclub.cu\" >http://gutl.jovenclub.cu/</a></p>"
            //"<p>Licensed under <a href= \"http://www.gnu.org/licenses/gpl.html\" >GNU General Public License version 3</a></p>"));
            "<p>Licensed under <a href= \"http://www.gnu.org/licenses/gpl.html\" >GNU General Public License version 3</a></p>"));

}



void View::MainWindow::createWidgets()
{
    createActions();
    createMenus();
    createToolBar();
    createStatusBar();

}


void View::MainWindow::createActions()
{
    _connectStorageAct = new QAction(trUtf8("Conectar Almacenamiento"), this);
    _connectStorageAct -> setIcon(QIcon(":/images/storageon.png"));
    _connectStorageAct -> setStatusTip(trUtf8("Conectar a la base de datos"));

    _disconnectStorageAct = new QAction(trUtf8("Desconectar Almacenamiento"), this);
    _disconnectStorageAct -> setIcon(QIcon(":/images/storageoff.png"));
    _disconnectStorageAct -> setStatusTip(trUtf8("Desconectar la base de datos"));

    _importStorageAct = new QAction(trUtf8("&Importar almacenamiento..."), this);
    _importStorageAct -> setIcon(QIcon(":/images/import.png"));
    _importStorageAct -> setStatusTip(trUtf8("Importar la base de datos"));

    _exportStorageAct = new QAction(trUtf8("&Exportar almacenamiento..."), this);
    _exportStorageAct -> setIcon(QIcon(":/images/export.png"));
    _exportStorageAct -> setStatusTip(trUtf8("Exportar la base de datos"));


    _exitAct = new QAction(trUtf8("&Cerrar"), this);
    _exitAct -> setIcon(QIcon(":/images/exit.png"));
    _exitAct -> setStatusTip(trUtf8("Salir de la aplicación"));

    _aboutAct = new QAction(trUtf8("Acerca de ..."), this);
    _aboutAct -> setIcon(QIcon(":/images/about.png"));
    _aboutAct -> setStatusTip(trUtf8("Ver información sobre la aplicación"));

    _aboutQtAct = new QAction(trUtf8("Acerca de &Qt"), this);
    _aboutQtAct -> setIcon(QIcon(":/images/aboutqt.png"));
    _aboutQtAct -> setStatusTip(trUtf8("Ver información sobre librerías Qt"));
}

void View::MainWindow::createMenus()
{
    _actionMenu = menuBar() -> addMenu(trUtf8("&Acciones"));
    _actionMenu -> addAction(_importStorageAct);
    _actionMenu -> addAction(_exportStorageAct);

    _actionMenu -> addSeparator();

    _actionMenu -> addAction(_connectStorageAct);
    _actionMenu -> addAction(_disconnectStorageAct);

    _actionMenu -> addSeparator();

    _actionMenu -> addAction(_exitAct);



    _helpMenu = menuBar() -> addMenu(trUtf8("A&yuda"));
    _helpMenu -> addAction(_aboutAct);
    _helpMenu -> addAction(_aboutQtAct);

}

void View::MainWindow::createToolBar()
{
    _actionToolBar = addToolBar(trUtf8("Accion"));
    _actionToolBar -> addAction(_importStorageAct);
    _actionToolBar -> addAction(_exportStorageAct);
    _actionToolBar -> setToolButtonStyle(Qt::ToolButtonIconOnly);

    _action1ToolBar = addToolBar(trUtf8("Accion1"));
    _action1ToolBar -> addAction(_connectStorageAct);
    _action1ToolBar -> addAction(_disconnectStorageAct);
    _action1ToolBar -> setToolButtonStyle(Qt::ToolButtonIconOnly);

    _helpToolBar = addToolBar(trUtf8("Ayuca"));
    _helpToolBar -> addAction(_aboutAct);
    _helpToolBar -> addAction(_aboutQtAct);
    _helpToolBar -> setToolButtonStyle(Qt::ToolButtonIconOnly);


}

void View::MainWindow::createStatusBar()
{
    _storageIconLabel = new QLabel;
    _storageStateLabel = new QLabel;

    statusBar() ->addWidget(_storageIconLabel);
    statusBar() ->addWidget(_storageStateLabel);
}

void View::MainWindow::createConnections()
{
    connect(_exitAct, SIGNAL(triggered()),
            this, SLOT(close()));
    connect(_connectStorageAct, SIGNAL(triggered()),
            this, SLOT(connectStorage()));
    connect(_importStorageAct, SIGNAL(triggered()),
            this, SLOT(importStorage()));
    connect(_exportStorageAct, SIGNAL(triggered()),
            this, SLOT(exportStorage()));
    connect(_disconnectStorageAct, SIGNAL(triggered()),
            this, SLOT(disconnectStorage()));
    connect(_aboutAct, SIGNAL(triggered()),
            this, SLOT(about()));
    connect(_aboutQtAct, SIGNAL(triggered()),
            qApp, SLOT(aboutQt()));
}



void View::MainWindow::setStorageConnected(bool connected)
{
    QString host = Persistence::Manager::readConfig("Host", "Storage/DBMS").toString();


    _connectStorageAct -> setEnabled(!connected);
    _disconnectStorageAct -> setEnabled(connected);
    _importStorageAct -> setEnabled(!connected);
    _exportStorageAct -> setEnabled(!connected);

    _storageIconLabel -> setPixmap(QPixmap(QString(":/images/storage%1.png")
                                           .arg(connected ? "on" : "off"))
                                   .scaled(QSize(16, 16)));
    _storageStateLabel -> setText(connected ? trUtf8("Conectado a %1").arg(host) : trUtf8("Almacenamiento desconectado"));
}


bool View::MainWindow::verifyImportStorage()
{
    return QMessageBox::warning(this, trUtf8("Verificar Importar Almacenamiento"),
                                       trUtf8("Debe saber que todos los datos almacenados serán eliminados.\n"
                                              "¿Esta seguro de que desea importar el almacenamiento?"),
                                       QMessageBox::Yes | QMessageBox::Default |
                                       QMessageBox::No) == QMessageBox::Yes;
}


bool View::MainWindow::verifyExit()
{
    return QMessageBox::question(this, trUtf8("Verificar Salida"),
                                 trUtf8("¿Está seguro de querer abandonar la aplicación?"),
                                 QMessageBox::Yes | QMessageBox::Default |
                                 QMessageBox::No) == QMessageBox::Yes;
}
